@@ -339,14 +339,13 @@ static const PreferenceDefault preference_defaults[] = {
 	  NULL, NULL,
 	  NULL
 	},
-	/* FIXME bugzilla.gnome.org 41245: Saved in pixels instead of in %? */
 	{ NAUTILUS_PREFERENCES_SIDEBAR_WIDTH,
 	  PREFERENCE_INTEGER,
 	  GINT_TO_POINTER (148)
 	},
 	{ NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER,
 	  PREFERENCE_BOOLEAN,
-	  GINT_TO_POINTER (FALSE)
+	  GINT_TO_POINTER (TRUE)
 	},
 	{ NAUTILUS_PREFERENCES_ENABLE_TABS,
 	  PREFERENCE_BOOLEAN,
@@ -378,6 +377,10 @@ static const PreferenceDefault preference_defaults[] = {
 	  PREFERENCE_BOOLEAN,
 	  GINT_TO_POINTER (TRUE)
 	},
+	{ NAUTILUS_PREFERENCES_START_WITH_EXTRA_PANE,
+	  PREFERENCE_BOOLEAN,
+	  GINT_TO_POINTER (FALSE)
+	},
 	{ NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_SAVED_GEOMETRY,
 	  PREFERENCE_STRING,
 	  ""
