@@ -26,43 +26,43 @@
 #ifndef NAUTILUS_TRASH_MONITOR_H
 #define NAUTILUS_TRASH_MONITOR_H
 
-#include <gtk/gtkobject.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
+#include <gtk/gtk.h>
+#include <gio/gio.h>
 
 typedef struct NautilusTrashMonitor NautilusTrashMonitor;
 typedef struct NautilusTrashMonitorClass NautilusTrashMonitorClass;
 typedef struct NautilusTrashMonitorDetails NautilusTrashMonitorDetails;
 
-#define NAUTILUS_TYPE_TRASH_MONITOR \
-	(nautilus_trash_monitor_get_type ())
+#define NAUTILUS_TYPE_TRASH_MONITOR nautilus_trash_monitor_get_type()
 #define NAUTILUS_TRASH_MONITOR(obj) \
-	(GTK_CHECK_CAST ((obj), NAUTILUS_TYPE_TRASH_MONITOR, NautilusTrashMonitor))
+  (G_TYPE_CHECK_INSTANCE_CAST ((obj), NAUTILUS_TYPE_TRASH_MONITOR, NautilusTrashMonitor))
 #define NAUTILUS_TRASH_MONITOR_CLASS(klass) \
-	(GTK_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_TRASH_MONITOR, NautilusTrashMonitorClass))
+  (G_TYPE_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_TRASH_MONITOR, NautilusTrashMonitorClass))
 #define NAUTILUS_IS_TRASH_MONITOR(obj) \
-	(GTK_CHECK_TYPE ((obj), NAUTILUS_TYPE_TRASH_MONITOR))
+  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NAUTILUS_TYPE_TRASH_MONITOR))
 #define NAUTILUS_IS_TRASH_MONITOR_CLASS(klass) \
-	(GTK_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_TRASH_MONITOR))
+  (G_TYPE_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_TRASH_MONITOR))
+#define NAUTILUS_TRASH_MONITOR_GET_CLASS(obj) \
+  (G_TYPE_INSTANCE_GET_CLASS ((obj), NAUTILUS_TYPE_TRASH_MONITOR, NautilusTrashMonitorClass))
 
 struct NautilusTrashMonitor {
-	GtkObject object;
+	GObject object;
 	NautilusTrashMonitorDetails *details;
 };
 
 struct NautilusTrashMonitorClass {
-	GtkObjectClass parent_class;
+	GObjectClass parent_class;
 
 	void (* trash_state_changed)		(NautilusTrashMonitor 	*trash_monitor,
 				      		 gboolean 		 new_state);
-	void (* check_trash_directory_added)	(NautilusTrashMonitor 	*trash_monitor,
-						 GnomeVFSVolume		*volume);
 };
 
-GtkType			nautilus_trash_monitor_get_type				(void);
+GType			nautilus_trash_monitor_get_type				(void);
 
 NautilusTrashMonitor   *nautilus_trash_monitor_get 				(void);
 gboolean		nautilus_trash_monitor_is_empty 			(void);
-GList *			nautilus_trash_monitor_get_trash_directories 		(void);
-void			nautilus_trash_monitor_add_new_trash_directories	(void);
+GIcon                  *nautilus_trash_monitor_get_icon                         (void);
+
+void		        nautilus_trash_monitor_add_new_trash_directories        (void);
 
 #endif
