@@ -27,7 +27,7 @@
 #ifndef NAUTILUS_MAIN_H
 #define NAUTILUS_MAIN_H
 
-#include <gtk/gtktypeutils.h>
+#include <gtk/gtk.h>
 
 void     nautilus_main_event_loop_register    (GtkObject *object);
 gboolean nautilus_main_is_event_loop_mainstay (GtkObject *object);
